using Microsoft.Xna.Framework;
using Microsoft.Xna.Framework.Content;
using Microsoft.Xna.Framework.Graphics;
using XQUEST.CameraSystem;
using XQUEST.GameObjectManagement;

namespace CameraDemo
{
  public class Box : ModelGameObject
  {
    // box texture, static to save memory since we are going to create 100 instances.
    public static Texture2D boxTexture;
    public static Model boxModel;

    public Box(Game game) : base(game, "ignore")
    {
      collideWithOtherObjects = true;
    }

    // load the box model
    public override void LoadContent(ContentManager content)
    {
      Model = boxModel;
    }
    
    public override bool CollidesWith(GameObject gameObject)
    {
      // Can only collide with duck.
      if (!(gameObject is Duck))
        return false;

      return base.CollidesWith(gameObject);
    }
    
    public override void CollisionOccured(GameObject collisionObject)
    {
      if (collisionObject is Duck)
      {
        // Box collided with duck.
        state = GameObjectState.Dead;
      }

      base.CollisionOccured(collisionObject);
    }

    public override void Draw(GameTime gameTime, SpriteBatch spriteBatch)
    {
      if (boxModel != null)
      {
        Matrix[] transforms = new Matrix[boxModel.Bones.Count];
        boxModel.CopyAbsoluteBoneTransformsTo(transforms);

        World = Matrix.CreateScale(Scale)*
                Matrix.CreateRotationX(Rotation.X)*
                Matrix.CreateRotationY(Rotation.Y)*
                Matrix.CreateRotationZ(Rotation.Z)*
                Matrix.CreateTranslation(Position);

        foreach (ModelMesh mesh in boxModel.Meshes)
        {
          foreach (BasicEffect effect in mesh.Effects)
          {
            effect.EnableDefaultLighting();

            effect.View = View;
            effect.Projection = Projection;
            effect.World = World*transforms[mesh.ParentBone.Index];
            effect.TextureEnabled = true;
            effect.Texture = boxTexture;
          }
          mesh.Draw();
        }
      }
    }
  }
}