using Microsoft.Xna.Framework;
using Microsoft.Xna.Framework.Input;
using XQUEST.CameraSystem;
using XQUEST.GameObjectManagement;
using XQUEST.Input;

namespace CameraDemo
{
  /// <summary>
  /// The duck class for the CameraDemo. Can move the duck using WASD on the keyboard.
  /// </summary>
  public class Duck : ModelGameObject
  {
    #region Fields

    private Camera3D currentCam;

    public Camera3D CurrentCam
    {
      get { return currentCam; }
      set { currentCam = value; }
    }

    #endregion

    #region Constructor

    public Duck(Game game, string modelName)
      : base(game, modelName)
    {
    }

    #endregion

    #region Overriden Interface

    public override void Initialize()
    {
      Scale = 5.0f;
      Position = new Vector3(0, 0, 0);
      MovementSpeed = 0.2f;
      TurnSpeed = 3f/60f;
    }
    // Update duck movement
    public override void Update(GameTime gameTime, InputManager input)
    {
      if (input.IsKeyOrButtonDown(Keys.A, Buttons.LeftThumbstickLeft))
      {
        // Rotate left.
        Rotation.Y = Rotation.Y + TurnSpeed % (MathHelper.TwoPi);
      }
      if (input.IsKeyOrButtonDown(Keys.D, Buttons.LeftThumbstickRight))
      {
        // Rotate right.
        Rotation.Y = Rotation.Y - TurnSpeed % (MathHelper.TwoPi);
      }
      if (input.IsKeyOrButtonDown(Keys.W, Buttons.LeftThumbstickUp))
      {
        Matrix forwardMovement = Matrix.CreateRotationY(Rotation.Y);
        Vector3 v = new Vector3(0, 0, MovementSpeed);
        v = Vector3.Transform(v, forwardMovement);
        Position.Z += v.Z;
        Position.X += v.X;
      }
      if (input.IsKeyOrButtonDown(Keys.S, Buttons.LeftThumbstickDown))
      {
        Matrix forwardMovement = Matrix.CreateRotationY(Rotation.Y);
        Vector3 v = new Vector3(0, 0, -MovementSpeed);
        v = Vector3.Transform(v, forwardMovement);
        Position.Z += v.Z;
        Position.X += v.X;
      }

      base.Update(gameTime, input);
    }

    #endregion
  }
}