using Microsoft.Xna.Framework;
using XQUEST.GameObjectManagement;
using XQUEST.Input;

namespace GameObjects_2D_Demo
{
  /// <summary>
  /// The bullet will shoot from the cannon.
  /// </summary>
  public class Bullet : SpriteGameObject
  {
    public Bullet(Game game, string textureName, Rectangle? sourceRect)
      : base(game, textureName, sourceRect)
    {
      // Recycle bullets when they die to minimize memory allocation.
      recycle = true;

      // Bullets can collide with dragons.
      collideWithOtherObjects = true;
    }

    public override void Update(GameTime gameTime, InputManager input)
    {
      // Set origin of bullet sprite to center, and double its size.
      Sprite.Origin = Sprite.Center;
      Sprite.Scale = new Vector2(2.0f);

      // Perform simple physics.
      Velocity += Acceleration;
      Position += Velocity;

      // Remove bullet if it goes out of screen.
      if (Position.X + Sprite.Width/2 > Game.GraphicsDevice.Viewport.Width ||
          Position.Y + Sprite.Height/2 > Game.GraphicsDevice.Viewport.Height ||
          Position.X < -Sprite.Width / 2 || Position.Y < -Sprite.Height / 2)
        state = GameObjectState.Dead;

      base.Update(gameTime, input);
    }

    public override void CollisionOccured(GameObject collisionObject)
    {
      // Remove bullet if it hits a dragon.
      if (collisionObject is Dragon)
      {
        state = GameObjectState.Dead;
      }
    }
  }
}