using Microsoft.Xna.Framework;
using Microsoft.Xna.Framework.Graphics;
using XQUEST.GameObjectManagement;
using XQUEST.Helpers;
using XQUEST.Input;
using XQUEST.SpriteAnimation;

namespace GameObjects_2D_Demo
{
  /// <summary>
  /// The dragon will float randomly around the screen.
  /// </summary>
  public class Dragon : SpriteGameObject
  {
    public Dragon(Game game, SpriteSheetInfo spriteSheetInfo)
      : base(game, spriteSheetInfo)
    {
      // The dragon can collide with the bullets.
      collideWithOtherObjects = true;

      // When a dragon dies, we just want to spawn it again somewhere
      // else on the screen, so we don't need recycling.
      recycle = false;

      // We need an animation object for describing how the frames will animate.
      // We want a delay of 0.1 seconds between each frame, and the frame sequence
      // will flip through the frames from the start to the end and back to the start
      // again.
      AddAnimation("flying",
                   new Animation(spriteSheetInfo.GetFrames(), 0.1f, true,
                                 new int[] {0, 1, 2, 3, 4, 3, 2, 1}));
    }

    public override void Initialize()
    {
      // Random position on screen.
      Position = RandomHelper.RandomVector2(0, 800 - 104, 0, 600 - 52);

      // Random velocity
      if (RandomHelper.RandomInt(0, 2) == 0)
        Velocity.X = -1;
      else
        Velocity.X = 1;

      if (RandomHelper.RandomInt(0, 2) == 0)
        Velocity.Y = -1;
      else
        Velocity.Y = 1;

      Velocity *= RandomHelper.RandomFloat(0.5f, 2.5f);

      base.Initialize();
    }

    public override void Update(GameTime gameTime, InputManager input)
    {
      // Apply simple physics.
      Velocity += Acceleration;
      Position += Velocity;

      // Bounce the dragon off the walls.
      if (Position.X + Sprite.Width >= Game.GraphicsDevice.Viewport.Width ||
          Position.X <= 0)
        Velocity.X = -Velocity.X;

      if (Position.Y + Sprite.Height >= Game.GraphicsDevice.Viewport.Height ||
          Position.Y <= 0)
        Velocity.Y = -Velocity.Y;

      // Always face it in the direction it travels.
      Sprite.Effects = Velocity.X > 0 ? SpriteEffects.FlipHorizontally : SpriteEffects.None;

      base.Update(gameTime, input);
    }

    /// <summary>
    /// This method is called when an object collides with
    /// the dragon.
    /// </summary>
    /// <param name="collisionObject"></param>
    public override void CollisionOccured(GameObject collisionObject)
    {
      // If the dragon is hit by a bullet, just re-initialize it.
      if (collisionObject is Bullet)
      {
        Initialize();
      }
    }
  }
}