using Microsoft.Xna.Framework;
using Microsoft.Xna.Framework.Graphics;
using Microsoft.Xna.Framework.Input;
using XQUEST.GameObjectManagement;
using XQUEST.Input;
using XQUEST.SpriteAnimation;

namespace GameObjects_2D_Demo
{
  /// <summary>
  /// This demo shows how to use SpriteGameObject and GameObjectManager
  /// to handle game objects in a 2D world using sprites.
  /// </summary>
  public class GameObject2DDemo : Game
  {
    private GraphicsDeviceManager graphics;
    private SpriteBatch spriteBatch;

    private InputManager input;
    private GameObjectManager gameObjectManager;

    private SpriteFont font;

    private bool usePerPixelCD = false;

    public GameObject2DDemo()
    {
      graphics = new GraphicsDeviceManager(this);
      Content.RootDirectory = "Content";
    }

    /// <summary>
    /// Allows the game to perform any initialization it needs to before starting to run.
    /// This is where it can query for any required services and load any non-graphic
    /// related content.  Calling base.Initialize will enumerate through any components
    /// and initialize them as well.
    /// </summary>
    protected override void Initialize()
    {
      input = new InputManager(this);
      Components.Add(input);

      gameObjectManager = new GameObjectManager(this);
      gameObjectManager.BlendMode = SpriteBlendMode.AlphaBlend;
      Components.Add(gameObjectManager);

      // Add the cannon.
      gameObjectManager.Add(new Cannon(this, "stick", null));

      // Add some dragons.
      SpriteSheetInfo dragonSheet =
        new SpriteSheetInfo("dragonsheet", 104, 52, 5, 1);
      gameObjectManager.Add(new Dragon(this, dragonSheet));
      gameObjectManager.Add(new Dragon(this, dragonSheet));
      gameObjectManager.Add(new Dragon(this, dragonSheet));
      gameObjectManager.Add(new Dragon(this, dragonSheet));
      gameObjectManager.Add(new Dragon(this, dragonSheet));

      base.Initialize();
    }

    /// <summary>
    /// LoadContent will be called once per game and is the place to load
    /// all of your content.
    /// </summary>
    protected override void LoadContent()
    {
      // Create a new SpriteBatch, which can be used to draw textures.
      spriteBatch = new SpriteBatch(GraphicsDevice);

      // Load a font for displaying text.
      font = Content.Load<SpriteFont>("Arial");
    }

    /// <summary>
    /// UnloadContent will be called once per game and is the place to unload
    /// all content.
    /// </summary>
    protected override void UnloadContent()
    {
      // TODO: Unload any non ContentManager content here
    }

    /// <summary>
    /// Allows the game to run logic such as updating the world,
    /// checking for collisions, gathering input, and playing audio.
    /// </summary>
    /// <param name="gameTime">Provides a snapshot of timing values.</param>
    protected override void Update(GameTime gameTime)
    {
      // Allows the game to exit
      if (GamePad.GetState(PlayerIndex.One).Buttons.Back == ButtonState.Pressed)
        Exit();

      if (input.Keyboard.IsKeyPressed(Keys.F1))
      {
        usePerPixelCD = !usePerPixelCD;
        foreach (GameObject go in gameObjectManager.ActiveGameObjects)
        {
          if (go is SpriteGameObject)
            ((SpriteGameObject) go).IsPerPixelCollisionDetection = usePerPixelCD;
        }
      }

      base.Update(gameTime);
    }

    /// <summary>
    /// This is called when the game should draw itself.
    /// </summary>
    /// <param name="gameTime">Provides a snapshot of timing values.</param>
    protected override void Draw(GameTime gameTime)
    {
      graphics.GraphicsDevice.Clear(Color.CornflowerBlue);

      spriteBatch.Begin();

      spriteBatch.DrawString(font,
                             "Per-pixel collision detection: " + usePerPixelCD +
                             " (F1 to toggle)", Vector2.Zero, Color.Black);

      spriteBatch.DrawString(font,
                             "Active Game Objects: " +
                             gameObjectManager.ActiveGameObjects.Count, new Vector2(0, 20),
                             Color.Black);

      spriteBatch.DrawString(font,
                             "Recycled game objects: " +
                             gameObjectManager.RecycleList.Count, new Vector2(0, 40),
                             Color.Black);

      spriteBatch.End();

      base.Draw(gameTime);
    }
  }
}