using Microsoft.Xna.Framework;
using Microsoft.Xna.Framework.Input;
using XQUEST.CameraSystem;
using XQUEST.GameObjectManagement;
using XQUEST.Input;

namespace Pointing_Demo
{
  /// <summary>
  /// This class is a ModelGameObject, used to implement the functionality
  /// of the ducks in the PointingDemo.
  /// </summary>
  public class Duck : ModelGameObject
  {
    #region Fields

    // rotationspeed, and number of times the duck has been clicked
    private float rotationSpeed;
    private int nClicked;
    
    private Camera3D currentCam;
    
    #endregion

    #region Properties
    public Camera3D CurrentCam
    {
      get { return currentCam; }
      set { currentCam = value; }
    }
    public int NClicked
    {
      get { return nClicked; }
      set { nClicked = value; }
    }
    #endregion


    #region Constructor

    public Duck(Game game, string modelName)
      : base(game, modelName)
    {
      Scale = 5.0f;
      Position = new Vector3(0, 0, 0);
      MovementSpeed = 0.2f;
      TurnSpeed = 3f / 60f;
      rotationSpeed = 0f;
    }

    #endregion

    #region Overriden Interface
    /// <summary>
    /// This method gets called when the duck is left clicked
    /// </summary>
    public override void LeftClicked()
    {
      // update click counter and increase / decrease rotation speed
      nClicked++;
      rotationSpeed += 1f/60f;
      base.LeftClicked();
    }
    /// <summary>
    /// This method gets called when the duck is right clicked
    /// </summary>
    public override void RightClicked()
    {
      // update click counter and increase / decrease rotation speed
      nClicked++;
      rotationSpeed -= 1f / 60f;
      base.RightClicked();
    }

    public override void Initialize()
    {
      base.Initialize();
    }

    public override void Update(GameTime gameTime, InputManager input)
    {
      // Rotate the duck if rotation speed != 0
      Rotation.Y += rotationSpeed;
      
      base.Update(gameTime, input);
    }

    #endregion
  }
}