using Microsoft.Xna.Framework;
using Microsoft.Xna.Framework.Graphics;
using Microsoft.Xna.Framework.Input;
using XQUEST.Helpers;
using XQUEST.Input;
using XQUEST.SpriteAnimation;

namespace TextRendering_Demo
{
  /// <summary>
  /// This is the main type for your game
  /// </summary>
  public class TextRenderingDemo : Game
  {
    #region Fields

    private GraphicsDeviceManager graphics;
    private SpriteBatch spriteBatch;

    private SpriteFont arial;
    private SpriteFont courier;
    private SpriteFont timesNewRoman;

    private InputManager input;

    private Vector2 shakeItPosition;
    private Vector2 scale;

    // AnimatedSprite object.
    private AnimatedSprite animatedSprite;
    // the dragon's position on screen.
    private Vector2 dragonPosition;
    // dragon's movement speed
    float movementSpeed = 3f;

    private int screenWidth, screenHeight;
    private bool isMovingRight = true;

    #endregion

    #region Constructor

    public TextRenderingDemo()
    {
      graphics = new GraphicsDeviceManager(this);
      Content.RootDirectory = "Content";
    }

    #endregion

    #region override methods
    /// <summary>
    /// Allows the game to perform any initialization it needs to before starting to run.
    /// This is where it can query for any required services and load any non-graphic
    /// related content.  Calling base.Initialize will enumerate through any components
    /// and initialize them as well.
    /// </summary>
    protected override void Initialize()
    {
      input = new InputManager(this);
      Components.Add(input);


      //flying dragon
      SpriteSheetInfo ssi = new SpriteSheetInfo("dragonsheet", 104, 52, 5, 1);

      Animation flyingAnimation =
        new Animation(ssi.GetFrames(), 0.1f, true, new int[] { 0, 1, 2, 3, 4, 3, 2, 1 });

      animatedSprite = new AnimatedSprite(ssi, flyingAnimation);
      animatedSprite.Origin = new Vector2(animatedSprite.Width / 2, animatedSprite.Height / 2);
      animatedSprite.Effects = SpriteEffects.FlipHorizontally;

      // initialized dragon's position to middle of game window
      dragonPosition = new Vector2(0, 500);

      // start with scale = 1;
      scale = Vector2.One;

      base.Initialize();
    }

    /// <summary>
    /// LoadContent will be called once per game and is the place to load
    /// all of your content.
    /// </summary>
    protected override void LoadContent()
    {
      screenWidth = GraphicsDevice.Viewport.Width;
      screenHeight = GraphicsDevice.Viewport.Height;

      // Create a new SpriteBatch, which can be used to draw textures.
      spriteBatch = new SpriteBatch(GraphicsDevice);

      arial = Content.Load<SpriteFont>("Arial");
      courier = Content.Load<SpriteFont>("Courier");
      timesNewRoman = Content.Load<SpriteFont>("TimesNewRoman");

      TextHelper.Initialize(arial, spriteBatch);

      // Load textures used by sprites.
      animatedSprite.LoadTexture(Content);
    }

    /// <summary>
    /// UnloadContent will be called once per game and is the place to unload
    /// all content.
    /// </summary>
    protected override void UnloadContent()
    {
      // TODO: Unload any non ContentManager content here
    }

    /// <summary>
    /// Allows the game to run logic such as updating the world,
    /// checking for collisions, gathering input, and playing audio.
    /// </summary>
    /// <param name="gameTime">Provides a snapshot of timing values.</param>
    protected override void Update(GameTime gameTime)
    {
      // Allows the game to exit
      if (GamePad.GetState(PlayerIndex.One).Buttons.Back == ButtonState.Pressed)
        Exit();

      if (input.Keyboard.IsKeyPressed(Keys.F1))
        TextHelper.Font = arial;
      if (input.Keyboard.IsKeyPressed(Keys.F2))
        TextHelper.Font = courier;
      if (input.Keyboard.IsKeyPressed(Keys.F3))
        TextHelper.Font = timesNewRoman;

      // Update the animation on the animated sprite.
      animatedSprite.UpdateAnimation(gameTime);

      UpdateDragonPosition();

      //updates shaking text position
      ShakeIt(new Vector2(550,550));

      //update scaleX
      ScaleIt();

      base.Update(gameTime);
    }

    /// <summary>
    /// This is called when the game should draw itself.
    /// </summary>
    /// <param name="gameTime">Provides a snapshot of timing values.</param>
    protected override void Draw(GameTime gameTime)
    {
      graphics.GraphicsDevice.Clear(Color.CornflowerBlue);

      spriteBatch.Begin();

      // Demonstrate normal text.
      TextHelper.DrawText("Normal text", new Vector2(screenWidth/2, 200),
                          Color.White);
      TextHelper.DrawTextCentered("Normal text, centered horizontally on screen",
                                  new Vector2(screenWidth / 2, 250), Color.Yellow);

      // Demonstrate shadowed text.
      TextHelper.DrawShadowedText("Text with shadow",
                                  new Vector2(screenWidth/2, 300), Color.Black,
                                  Color.White);

      TextHelper.DrawShadowedTextCentered("Text with shadow, centered horizontally on screen",
                                  new Vector2(screenWidth / 2, 350), Color.Red,
                                  Color.Yellow);

      // Info
      TextHelper.DrawText("Font:", Vector2.Zero, Color.Black);
      TextHelper.DrawText("[F1] : Arial", new Vector2(0, 20), Color.Black);
      TextHelper.DrawText("[F2] : Courier", new Vector2(0, 40), Color.Black);
      TextHelper.DrawText("[F3] : Times New Roman", new Vector2(0, 60), Color.Black);


      // draw dragon
      animatedSprite.Draw(spriteBatch, dragonPosition);

      // draw text above dragon
      TextHelper.DrawText("I\'m moving!", dragonPosition+new Vector2(0,-50), Color.Pink);

      // draw shaking text
      TextHelper.DrawText("I\'m shaking!", shakeItPosition, Color.Pink);

      // draw scaling text
      TextHelper.DrawTextCentered("I\'m being scaled!", new Vector2(450, 50), Color.Pink, scale);

      spriteBatch.End();
         
      base.Draw(gameTime);
    }
    #endregion

    #region Methods

    public void UpdateDragonPosition()
    {
      if(isMovingRight)
      {
        dragonPosition.X += movementSpeed;
      }
      else
      {
        dragonPosition.X -= movementSpeed;
      }

      if(dragonPosition.X > screenWidth)
      {
        dragonPosition.X -= movementSpeed;
        isMovingRight = false;
        animatedSprite.Effects = SpriteEffects.None;
      }
      else if (dragonPosition.X < 0)
      {
        dragonPosition.X += movementSpeed;
        isMovingRight = true;
        animatedSprite.Effects = SpriteEffects.FlipHorizontally;
      }
 
    }

    public void ScaleIt()
    {
      scale.X += RandomHelper.RandomFloat(-0.1f, 0.1f);
      scale.Y += RandomHelper.RandomFloat(-0.1f, 0.1f);

      scale.X = MathHelper.Clamp(scale.X, 0.1f, 3f);
      scale.Y = MathHelper.Clamp(scale.Y, 0.1f, 3f);
    }

    public Vector2 ShakeIt(Vector2 stayCloseToPosition)
    {
      float maxOffset = 10f;
      float distance = 2f;
      shakeItPosition += GetRandomVector2()*distance;

      shakeItPosition.X = MathHelper.Clamp(shakeItPosition.X, stayCloseToPosition.X - maxOffset, stayCloseToPosition.X + maxOffset);
      shakeItPosition.Y = MathHelper.Clamp(shakeItPosition.Y, stayCloseToPosition.Y - maxOffset, stayCloseToPosition.Y + maxOffset);

      return shakeItPosition;
    }

    public Vector2 GetRandomVector2()
    {
      return new Vector2(RandomHelper.RandomFloat(-1,1), RandomHelper.RandomFloat(-1,1));
    }

    #endregion
  }
}