using Microsoft.Xna.Framework;
using XQUEST.GameObjectManagement;

namespace XQUEST.CameraSystem
{
  /// <summary>
  /// Base class for 3D cameras.
  /// </summary>
  public abstract class Camera3D : Camera
  {
    #region Fields

    protected float fieldOfView;
    protected float nearPlane;
    protected float farPlane;

    public Vector3 Position;

    protected Matrix viewMatrix;
    protected Matrix projectionMatrix;

    #endregion

    #region Constructors

    protected Camera3D(Game game)
      : this(game, CameraSplitScreenSetting.Fullscreen)
    {
    }

    protected Camera3D(Game game, CameraSplitScreenSetting cameraSplitScreenSetting)
      : base(game, cameraSplitScreenSetting)
    {
      // Set up default values for projection
      fieldOfView = MathHelper.PiOver2;
      nearPlane = 0.01f;
      farPlane = 5000.0f;

      Position = Vector3.Zero;

      viewMatrix = Matrix.CreateLookAt(Position, Vector3.Zero, Vector3.Up);
      projectionMatrix =
        Matrix.CreatePerspectiveFieldOfView(fieldOfView, viewport.AspectRatio,
                                            nearPlane, farPlane);
    }

    #endregion

    #region Properties

    /// <summary>
    /// Gets or set the field of view for this camera.
    /// </summary>
    public float FieldOfView
    {
      get { return fieldOfView; }
      set { fieldOfView = value; }
    }
    
    /// <summary>
    /// Gets or sets the location of the near plane of this camera.
    /// </summary>
    public float NearPlane
    {
      get { return nearPlane; }
      set { nearPlane = value; }
    }

    /// <summary>
    /// Gets or sets the location of the far plane for this camera.
    /// </summary>
    public float FarPlane
    {
      get { return farPlane; }
      set { farPlane = value; }
    }

    #endregion

    #region Overridden Interface

    /// <summary>
    /// Gets the projection matrix of this camera.
    /// </summary>
    public override Matrix Projection
    {
      get { return projectionMatrix; }
    }

    /// <summary>
    /// Gets the view matrix of this camera.
    /// </summary>
    public override Matrix View
    {
      get { return viewMatrix; }
    }

    #endregion
  }
}