using Microsoft.Xna.Framework;
using Microsoft.Xna.Framework.Input;
using XQUEST.Input;

namespace XQUEST.CameraSystem
{
  /// <summary>
  /// This Camera follows a game object, but can be freely rotated
  /// to look around.
  /// </summary>
  public class FreeThirdPersonCamera : ThirdPersonCamera
  {
    #region Fields

    private float rotationSpeed;

    #endregion

    #region Constructors

    public FreeThirdPersonCamera(Game game)
      : this(game, CameraSplitScreenSetting.Fullscreen)
    {
    }

    public FreeThirdPersonCamera(Game game,
                                 CameraSplitScreenSetting cameraSplitScreenSetting)
      : base(game, cameraSplitScreenSetting)
    {
      // Set rates in world units per 1/60th second (the default fixed-step interval).
      rotationSpeed = 6f/60f;
    }

    #endregion

    #region Properties

    /// <summary>
    /// Gets or sets the speed of camera rotation around the game object
    /// </summary>
    public float RotationSpeed
    {
      get { return rotationSpeed; }
      set { rotationSpeed = value; }
    }

    #endregion

    #region Methods

    /// <summary>
    /// Queries and interprets input from keyboard and mouse.
    /// </summary>
    protected override void UpdateCameraInput()
    {
      InputManager input =
        (InputManager) Game.Services.GetService(typeof (InputManager));

      // KEYBOARD INPUT
      if ((input.Keyboard.IsKeyDown(Keys.LeftAlt) &&
           input.Keyboard.IsKeyDown(Keys.Down)) ||
          input.GamePad.IsButtonDown(Buttons.LeftShoulder))
      {
        deltaY -= heightChangeSpeed;
      }
      else if ((input.Keyboard.IsKeyDown(Keys.LeftAlt) &&
                input.Keyboard.IsKeyDown(Keys.Up)) ||
               input.GamePad.IsButtonDown(Buttons.RightShoulder))
      {
        deltaY += heightChangeSpeed;
      }

      if (input.Keyboard.IsKeyDown(Keys.LeftAlt) &&
          input.Keyboard.IsKeyDown(Keys.Left))
      {
        cameraYaw += rotationSpeed%MathHelper.TwoPi;
      }
      if (input.Keyboard.IsKeyDown(Keys.LeftAlt) &&
          input.Keyboard.IsKeyDown(Keys.Right))
      {
        cameraYaw -= rotationSpeed%MathHelper.TwoPi;
      }

      // MOUSE INPUT
#if !XBOX
      if (input.Mouse.IsRightButtonDown)
      {
        if (input.Mouse.PositionY < (Viewport.Height/2))
        {
          deltaZ -= zoomChangeSpeed;
        }
        else if (input.Mouse.PositionY > (Viewport.Height/2))
        {
          deltaZ += zoomChangeSpeed;
        }

        if (input.Mouse.PositionX < (Viewport.Width/2))
        {
          cameraYaw -= rotationSpeed%(MathHelper.TwoPi);
        }
        else if (input.Mouse.PositionX > (Viewport.Width/2))
        {
          cameraYaw += rotationSpeed%(MathHelper.TwoPi);
        }

        if (input.Mouse.ScrollWheelDelta != 0)
        {
          if (input.Mouse.ScrollWheelDelta < 0)
          {
            deltaZ += zoomChangeSpeed;
          }
          else if (input.Mouse.ScrollWheelDelta > 0)
          {
            deltaZ -= zoomChangeSpeed;
          }
        }

        // Set mouse position to center so that it wont stop rotating when it reaches the edge of the screen.
        Mouse.SetPosition(Viewport.Width/2, Viewport.Height/2);
      }
#endif

      deltaY = MathHelper.Clamp(deltaY, minCameraPositionY - defaultCameraPosition.Y,
                                maxCameraPositionY - defaultCameraPosition.Y);
      deltaZ = MathHelper.Clamp(deltaZ, minCameraPositionZ - defaultCameraPosition.Z,
                                maxCameraPositionZ - defaultCameraPosition.Z);
    }

    #endregion
  }
}