using Microsoft.Xna.Framework;

namespace XQUEST.GameObjectManagement.BoundingVolumes
{
  /// <summary>
  /// Interface for bounding volumes.
  /// </summary>
  public interface IBoundingVolume
  {
    /// <summary>
    /// Checks whether this bounding volume intersects another
    /// bounding volume.
    /// </summary>
    /// <param name="boundingVolume">Another bounding volume.</param>
    /// <returns>
    /// Whether or not this bounding volume intersects 
    /// another bounding volume.
    /// </returns>
    bool Intersects(IBoundingVolume boundingVolume);

    /// <summary>
    /// Checks whether this bounding volume is hit by a ray.
    /// </summary>
    /// <param name="ray">A ray to check.</param>
    /// <returns>The distance from the ray position to the bounding volume, or null if the ray
    /// does not hit the bounding volume.</returns>
    float? RayIntersects(Ray ray);

    /// <summary>
    /// Checks whether this bounding volume is intersected by a plane.
    /// </summary>
    /// <param name="plane">A plane to check.</param>
    /// <returns></returns>
    PlaneIntersectionType PlaneIntersects(Plane plane);

    /// <summary>
    /// The width of the bounding volume.
    /// </summary>
    float Width { get; }

    /// <summary>
    /// The height of the bounding volume.
    /// </summary>
    float Height { get; }

    /// <summary>
    /// The depth of the bounding volume.
    /// </summary>
    float Depth { get; }
  }
}