using Microsoft.Xna.Framework;

namespace XQUEST.GameObjectManagement.BoundingVolumes
{
  /// <summary>
  /// Wrapper around the XNA BoundingBox struct.
  /// </summary>
  public struct XQBoundingBox : IBoundingVolume
  {
    // The XNA BoundingBox struct.
    public BoundingBox BoundingBox;

    /// <summary>
    /// Creates a new XQBoundingBox.
    /// </summary>
    /// <param name="min">The minimum point the bounding box includes.</param>
    /// <param name="max">The maximum point the bounding box includes.</param>
    public XQBoundingBox(Vector3 min, Vector3 max)
    {
      BoundingBox = new BoundingBox(min, max);
    }

    /// <summary>
    /// Creates a new XQBoundingBox.
    /// </summary>
    /// <param name="box">An XNA BoundingBox struct.</param>
    public XQBoundingBox(BoundingBox box)
    {
      BoundingBox = box;
    }

    public bool Intersects(IBoundingVolume boundingVolume)
    {
      if (boundingVolume is XQBoundingBox)
        return ((XQBoundingBox) boundingVolume).BoundingBox.Intersects(BoundingBox);

      if (boundingVolume is XQBoundingSphere)
        return
          ((XQBoundingSphere) boundingVolume).BoundingSphere.Intersects(BoundingBox);

      if (boundingVolume is XQBoundingFrustum)
        return
          ((XQBoundingFrustum) boundingVolume).BoundingFrustum.Intersects(BoundingBox);

      return false;
    }

    public float? RayIntersects(Ray ray)
    {
      return ray.Intersects(BoundingBox);
    }

    public PlaneIntersectionType PlaneIntersects(Plane plane)
    {
      return plane.Intersects(BoundingBox);
    }

    public float Width
    {
      get { return BoundingBox.Max.X - BoundingBox.Min.X; }
    }

    public float Height
    {
      get { return BoundingBox.Max.Y - BoundingBox.Min.Y; }
    }

    public float Depth
    {
      get { return BoundingBox.Max.Z - BoundingBox.Min.Z; }
    }
  }
}