using System;
using Microsoft.Xna.Framework;

namespace XQUEST.GameObjectManagement.BoundingVolumes
{
  /// <summary>
  /// Wrapper around the XNA BoundingFrustum struct.
  /// </summary>
  public struct XQBoundingFrustum : IBoundingVolume
  {
    // The XNA BoundingFrustum struct.
    public BoundingFrustum BoundingFrustum;

    /// <summary>
    /// Creates a new XQBoundingFrustum.
    /// </summary>
    /// <param name="value">Combined matrix that usually takes view x projection matrix.</param>
    public XQBoundingFrustum(Matrix value)
    {
      BoundingFrustum = new BoundingFrustum(value);
    }

    /// <summary>
    /// Creates a new XQBoundingFrustum.
    /// </summary>
    /// <param name="boundingFrustum">The XNA BoundingFrustum struct.</param>
    public XQBoundingFrustum(BoundingFrustum boundingFrustum)
    {
      BoundingFrustum = boundingFrustum;
    }

    public bool Intersects(IBoundingVolume boundingVolume)
    {
      if (boundingVolume is XQBoundingBox)
        return
          ((XQBoundingBox) boundingVolume).BoundingBox.Intersects(BoundingFrustum);

      if (boundingVolume is XQBoundingSphere)
        return
          ((XQBoundingSphere) boundingVolume).BoundingSphere.Intersects(
            BoundingFrustum);

      if (boundingVolume is XQBoundingFrustum)
        return
          ((XQBoundingFrustum) boundingVolume).BoundingFrustum.Intersects(
            BoundingFrustum);

      return false;
    }

    public float? RayIntersects(Ray ray)
    {
      return ray.Intersects(BoundingFrustum);
    }

    public PlaneIntersectionType PlaneIntersects(Plane plane)
    {
      return plane.Intersects(BoundingFrustum);
    }

    public float Width
    {
      get { throw new System.NotImplementedException(); }
    }

    public float Height
    {
      get { throw new System.NotImplementedException(); }
    }

    public float Depth
    {
      get { throw new System.NotImplementedException(); }
    }

    public ContainmentType Contains(IBoundingVolume boundingVolume)
    {
      if (boundingVolume is XQBoundingBox)
        return
          BoundingFrustum.Contains(((XQBoundingBox) boundingVolume).BoundingBox);

      if (boundingVolume is XQBoundingSphere)
        return
          BoundingFrustum.Contains(((XQBoundingSphere) boundingVolume).BoundingSphere);

      if (boundingVolume is XQBoundingFrustum)
        return
          BoundingFrustum.Contains(
            ((XQBoundingFrustum) boundingVolume).BoundingFrustum);

      return ContainmentType.Disjoint;
    }
  }
}