using Microsoft.Xna.Framework;

namespace XQUEST.GameObjectManagement.BoundingVolumes
{
  /// <summary>
  /// Wrapper around the XNA Rectangle struct.
  /// </summary>
  public struct XQBoundingRectangle : IBoundingVolume
  {
    // The XNA Rectangle struct.
    public Rectangle Rectangle;

    // Creates a new XQBoundingRectangle.
    public XQBoundingRectangle(int x, int y, int width, int height)
    {
      Rectangle = new Rectangle(x, y, width, height);
    }
    
    /// <summary>
    /// Creates a new XQBoundingRectangle.
    /// </summary>
    /// <param name="rectangle">An XNA Rectangle.</param>
    public XQBoundingRectangle(Rectangle rectangle)
    {
      Rectangle = rectangle;
    }

    public bool Intersects(IBoundingVolume boundingVolume)
    {
      // Only allow for Rectangle <-> Rectangle collision.
      if (!(boundingVolume is XQBoundingRectangle))
        return false;

      return ((XQBoundingRectangle) boundingVolume).Rectangle.Intersects(Rectangle);
    }

    /// <summary>
    /// This method is not supported on this bounding volume.
    /// </summary>
    public float? RayIntersects(Ray ray)
    {
      throw new System.NotSupportedException("A ray cannot intersect a 2D rectangle!");
    }

    /// <summary>
    /// This method is not supported on this bounding volume.
    /// </summary>
    public PlaneIntersectionType PlaneIntersects(Plane plane)
    {
      throw new System.NotImplementedException("A plane cannot intersect a 2D rectangle!");
    }

    public float Width
    {
      get { return Rectangle.Width; }
    }

    public float Height
    {
      get { return Rectangle.Height; }
    }

    public float Depth
    {
      get { return 0.0f; }
    }
  }
}