using Microsoft.Xna.Framework;

namespace XQUEST.GameObjectManagement.BoundingVolumes
{
  /// <summary>
  /// Wrapper around the XNA BoundingSphere struct.
  /// </summary>
  public struct XQBoundingSphere : IBoundingVolume
  {
    // The XNA BoundingSphere struct.
    public BoundingSphere BoundingSphere;

    /// <summary>
    /// Creates a new XQBoundingSphere.
    /// </summary>
    /// <param name="center">Center point of the sphere.</param>
    /// <param name="radius">Radius of the sphere</param>
    public XQBoundingSphere(Vector3 center, float radius)
    {
      BoundingSphere = new BoundingSphere(center, radius);
    }

    /// <summary>
    /// Creates a new XQBoundingSphere.
    /// </summary>
    /// <param name="boundingSphere">An XNA BoundingSphere to wrap.</param>
    public XQBoundingSphere(BoundingSphere boundingSphere)
    {
      BoundingSphere = boundingSphere;
    }

    public bool Intersects(IBoundingVolume boundingVolume)
    {
      if (boundingVolume is XQBoundingBox)
        return
          ((XQBoundingBox) boundingVolume).BoundingBox.Intersects(BoundingSphere);

      if (boundingVolume is XQBoundingSphere)
        return
          ((XQBoundingSphere) boundingVolume).BoundingSphere.Intersects(
            BoundingSphere);

      if (boundingVolume is XQBoundingFrustum)
        return
          ((XQBoundingFrustum) boundingVolume).BoundingFrustum.Intersects(
            BoundingSphere);

      return false;
    }

    public float? RayIntersects(Ray ray)
    {
      return ray.Intersects(BoundingSphere);
    }

    public PlaneIntersectionType PlaneIntersects(Plane plane)
    {
      return plane.Intersects(BoundingSphere);
    }

    public float Width
    {
      get { return BoundingSphere.Radius*2; }
    }

    public float Height
    {
      get { return BoundingSphere.Radius * 2; }
    }

    public float Depth
    {
      get { return BoundingSphere.Radius * 2; }
    }
  }
}