using System.Collections.Generic;
using Microsoft.Xna.Framework;
using XQUEST.CameraSystem;
using XQUEST.Input;

namespace XQUEST.GameObjectManagement
{
#if !XBOX
  public class ClickableManager : GameComponent
  {
    #region Fields and Properties

    private InputManager inputManager;
    private CameraManager cameraManager;
    private List<ModelGameObject> modelGOs;

    public List<ModelGameObject> ModelGameObjects
    {
      get { return modelGOs; }
      set { modelGOs = value; }
    }

    public int Count
    {
      get { return modelGOs.Count; }
    }

    #endregion

    #region Constructor

    public ClickableManager(Game game)
      : base(game)
    {
      modelGOs = new List<ModelGameObject>();
      game.Services.AddService(typeof (ClickableManager), this);
    }

    #endregion

    #region Public Methods

    public void Add(ModelGameObject modelGO)
    {
      modelGOs.Add(modelGO);
    }

    public void Remove(ModelGameObject modelGO)
    {
      modelGOs.Remove(modelGO);
    }

    // CalculateCursorRay Calculates a world space ray starting at the camera's
    // "eye" and pointing in the direction of the cursor. Viewport.Unproject is used
    // to accomplish this.
    private Ray CalculateCursorRay()
    {
      Matrix projection = cameraManager.ActiveCamera.Projection;
      Matrix view = cameraManager.ActiveCamera.View;

      Vector3 rayStart =
        Game.GraphicsDevice.Viewport.Unproject(
          new Vector3(inputManager.Mouse.PositionX, inputManager.Mouse.PositionY,
                      0.01f),
          projection, view, Matrix.Identity);

      Vector3 rayEnd =
        Game.GraphicsDevice.Viewport.Unproject(
          new Vector3(inputManager.Mouse.PositionX, inputManager.Mouse.PositionY,
                      0.99f),
          projection, view, Matrix.Identity);

      return new Ray(rayStart, Vector3.Normalize(rayEnd - rayStart));
    }

    #endregion

    #region Overriden Interface

    public override void Initialize()
    {
      inputManager = (InputManager) Game.Services.GetService(typeof (InputManager));
      cameraManager =
        (CameraManager) Game.Services.GetService(typeof (CameraManager));

      base.Initialize();
    }

    public override void Update(GameTime gameTime)
    {
      if (inputManager.Mouse.IsLeftButtonPressed ||
          inputManager.Mouse.IsRightButtonPressed)
      {
        float minDistance = 0;
        float? distance;
        int nearest = -1;
        bool first = true;

        Ray cursorRay = CalculateCursorRay();
        foreach (ModelGameObject modelGO in modelGOs)
        {
          distance = modelGO.BoundingVolume.RayIntersects(cursorRay);

          if (distance != null)
          {
            Vector3 intersectionPoint = cursorRay.Position +
                                        cursorRay.Direction*(float) distance;
            Vector3 distanceToCamera = cursorRay.Position - intersectionPoint;

            if (first)
            {
              minDistance = distanceToCamera.Length();
              nearest = modelGOs.IndexOf(modelGO);
              first = false;
            }
            else
            {
              if (distanceToCamera.Length() < minDistance)
              {
                minDistance = distanceToCamera.Length();
                nearest = modelGOs.IndexOf(modelGO);
              }
            }
          }
        }

        if (nearest != -1)
        {
          if (inputManager.Mouse.IsLeftButtonPressed)
          {
            modelGOs[nearest].LeftClicked();
          }
          else
          {
            modelGOs[nearest].RightClicked();
          }
        }
      }
    }

    #endregion
  }
#endif
}