using System;
using Microsoft.Xna.Framework;
using Microsoft.Xna.Framework.Graphics;

namespace XQUEST.Misc
{
  public class FPSCounter : DrawableGameComponent
  {
    private readonly string fontContentPath;
    private int frameCount;
    private int fps;
    private TimeSpan elapsedTime;

    private SpriteFont fpsFont;
    private SpriteBatch spriteBatch;

    private Vector2 position;
    private Color color;

    public Vector2 Position
    {
      get { return position; }
      set { position = value; }
    }

    public int Fps
    {
      get { return fps; }
    }

    public Color Color
    {
      get { return color; }
      set { color = value; }
    }

    public FPSCounter(Game game, string fontContentPath) : base(game)
    {
      this.fontContentPath = fontContentPath;
      frameCount = 0;
      fps = 0;

      position = new Vector2(80.0f, 60.0f);
      color = Color.White;
    }

    protected override void LoadContent()
    {
      fpsFont = Game.Content.Load<SpriteFont>(fontContentPath);
      spriteBatch = new SpriteBatch(GraphicsDevice);
      base.LoadContent();
    }

    public override void Update(GameTime gameTime)
    {
      elapsedTime += gameTime.ElapsedGameTime;
      if (elapsedTime > TimeSpan.FromSeconds(1))
      {
        elapsedTime -= TimeSpan.FromSeconds(1);
        fps = frameCount;
        frameCount = 0;
      }
      base.Update(gameTime);
    }

    public override void Draw(GameTime gameTime)
    {
      spriteBatch.Begin();
      spriteBatch.DrawString(fpsFont, string.Format("FPS: {0}", fps), position, color, 0.0f,
                             Vector2.Zero, 1.0f, SpriteEffects.None, 1.0f);
      spriteBatch.End();

      base.Draw(gameTime);
      frameCount++;
    }
  }
}