using System.Collections.Generic;
using Microsoft.Xna.Framework;

namespace XQUEST.SpriteAnimation
{
  /// <summary>
  /// Holds information about a sprite sheet.
  /// </summary>
  public struct SpriteSheetInfo
  {
    #region Public Fields

    public string TextureName;
    public int FrameWidth;
    public int FrameHeight;

    public int FramesPerRow;
    public int FramesPerColumn;

    public int RowSpacing;
    public int ColumnSpacing;

    #endregion

    #region Constructors

    public SpriteSheetInfo(string textureName, int frameWidth, int frameHeight,
                       int framesPerRow, int framesPerColumn, int rowSpacing,
                       int columnSpacing)
    {
      TextureName = textureName;
      FrameWidth = frameWidth;
      FrameHeight = frameHeight;
      FramesPerRow = framesPerRow;
      FramesPerColumn = framesPerColumn;
      RowSpacing = rowSpacing;
      ColumnSpacing = columnSpacing;
    }
    
    public SpriteSheetInfo(string textureName, int frameWidth, int frameHeight,
                       int framesPerRow, int framesPerColumn)
    {
      TextureName = textureName;
      FrameWidth = frameWidth;
      FrameHeight = frameHeight;
      FramesPerRow = framesPerRow;
      FramesPerColumn = framesPerColumn;
      ColumnSpacing = 0;
      RowSpacing = 0;
    }

    #endregion

    #region Public Methods

    /// <summary>
    /// Creates a list of rectangles specifying
    /// the individual frames of the sprite sheet.
    /// The rectangles are order left-to-right, top-to-bottom.
    /// </summary>
    /// <returns></returns>
    public List<Rectangle> GetFrames()
    {
      List<Rectangle> frames = new List<Rectangle>();
      int width = (FramesPerRow*FrameWidth) + (RowSpacing*(FramesPerRow - 1));
      int height = (FramesPerColumn*FrameHeight) + (ColumnSpacing*(FramesPerColumn- 1));

      for (int y = 0; y < height; y += FrameHeight + RowSpacing)
      {
        for (int x = 0; x < width; x += FrameWidth + ColumnSpacing)
        {
          frames.Add(new Rectangle(x, y, FrameWidth, FrameHeight));
        }
      }
      return frames;
    }

    #endregion
  }
}